# Font Mapper exceptions
class NoMapForOriginException(Exception):
    pass


# General file type handler exceptions
class UnsupportedMapToException(Exception):
    pass


# Txt File handler exceptions
class TxtAutoModeException(Exception):
    pass



import re
import json
import html
from .exceptions import NoMapForOriginException
from .preetimapper import convert as pmconvert


class FontMapper:
    def __init__(self, map_json):
        self.all_rules = json.load(open(map_json, 'r', encoding='utf-8'))
        self.supported_maps = list(self.all_rules.keys())
        self.supported_maps.append("Unicode")
        self.known_devanagari_unicode_fonts = ["Kalimati", "Mangal", "Noto Sans Devanagari"]

    def map_to_unicode(self, string, from_font="Preeti", unescape_html_input=False, escape_html_output=False):
        if not from_font.lower() == "unicode":
            if from_font in self.supported_maps:
                if unescape_html_input:
                    string = html.unescape(string)

                rules = self.all_rules[from_font]['rules']
                split_pattern = re.compile(r'(\s+|\S+)')
                mapped_string = ''

                for word in re.findall(split_pattern, string):
                    for rule in rules['pre-rules']:
                        word = re.sub(re.compile(rule[0]), rule[1], word)
                    mapped_word = ''.join(rules['character-map'].get(character, character) for character in word)
                    for rule in rules['post-rules']:
                        mapped_word = re.sub(re.compile(rule[0]), rule[1], mapped_word)
                    mapped_string = mapped_string + mapped_word
                if escape_html_output:
                    return html.escape(mapped_string)
                else:
                    return mapped_string
            else:
                raise NoMapForOriginException
        else:
            return string

    def map_to_preeti(self, string, from_font="Preeti", unescape_html_input=False, escape_html_output=False):
        if unescape_html_input:
            string = html.unescape(string)
        if not from_font.lower() == "preeti":
            # Map the string to unicode first
            unicode_mapped_string = self.map_to_unicode(string, from_font)
            # Now map the unicode to preeti
            mapped_string = pmconvert(unicode_mapped_string)
            if escape_html_output:
                return html.escape(mapped_string)
            else:
                return mapped_string
        else:
            return string



from .fontmapper import FontMapper
from .exceptions import TxtAutoModeException, UnsupportedMapToException

class TxtHandler:
    
    def __init__(self, rules_file):
        self.mapper = FontMapper(rules_file)
        self.supported_ttf_fonts = self.mapper.supported_maps
    
    def detect_used_fonts(self, docx_file_path):
        return []

    def map_fonts(self, orginal_file_path, output_file_path="mapped.txt", from_font="Preeti", to_font="unicode", components=[], known_unicode_fonts=[]):
        if from_font != "auto":
            with open(orginal_file_path, "r") as orginal_file:
                lines_orginal = orginal_file.readlines()
            output_file =  open(output_file_path, "a")
            for line in lines_orginal:
                if to_font.lower() == "unicode":
                    output_file.write(self.mapper.map_to_unicode(line, from_font, False))
                elif to_font.lower() == "preeti":
                    output_file.write(self.mapper.map_to_preeti(line, from_font, False))
                else:
                    raise UnsupportedMapToException
            output_file.close()
        else:
            raise TxtAutoModeException
        return True




# This part of code is derived from "https://github.com/globalpolicy/UnicodeToPreeti" 
# and is licensed under MIT license


# MIT License

# Copyright (c) 2017 Global Policy

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

unicodeToPreetiDict = \
    {
        "अ": "c",
        "आ": "cf",
        "ा": "f",
        "इ": "O",
        "ई": "O{",
        "र्": "{",
        "उ": "p",
        "ए": "P",
        "े": "]",
        "ै": "}",
        "ो": "f]",
        "ौ": "f}",
        "ओ": "cf]",
        "औ": "cf}",
        "ं": "+",
        "ँ": "F",
        "ि": "l",
        "ी": "L",
        "ु": "'",
        "ू": '"',
        "क": "s",
        "ख": "v",
        "ग": "u",
        "घ": "3",
        "ङ": "ª",
        "च": "r",
        "छ": "5",
        "ज": "h",
        "झ": "´",
        "ञ": "`",
        "ट": "6",
        "ठ": "7",
        "ड": "8",
        "ढ": "9",
        "ण": "0f",
        "त": "t",
        "थ": "y",
        "द": "b",
        "ध": "w",
        "न": "g",
        "प": "k",
        "फ": "km",
        "ब": "a",
        "भ": "e",
        "म": "d",
        "य": "o",
        "र": "/",
        "रू": "?",
        "ृ": "[",
        "ल": "n",
        "व": "j",
        "स": ";",
        "श": "z",
        "ष": "if",
        "ज्ञ": "1",
        "ह": "x",
        "१": "!",
        "२": "@",
        "३": "#",
        "४": "$",
        "५": "%",
        "६": "^",
        "७": "&",
        "८": "*",
        "९": "(",
        "०": ")",
        "।": ".",
        "्": "\\",
        "ऊ": "pm",
        "-": " ",
        "(": "-",
        ")": "_"
    }


def normalizeUnicode(unicodetext):
    index = -1
    normalized = ''
    while index + 1 < len(unicodetext):
        index += 1
        character = unicodetext[index]
        try:
            try:
                if character != 'र':  # for aadha akshars
                    if unicodetext[index + 1] == '्' and unicodetext[index + 2] != ' ' and unicodetext[index+2] != '।' \
                            and unicodetext[index + 2] != ',':
                        if unicodetext[index + 2] != 'र':
                            if unicodeToPreetiDict[character] in list('wertyuxasdghjkzvn'):
                                normalized += chr(ord(unicodeToPreetiDict[character]) - 32)
                                index += 1
                                continue
                            elif character == 'स':
                                normalized += ':'
                                index += 1
                                continue
                            elif character == 'ष':
                                normalized += 'i'
                                index += 1
                                continue
                if unicodetext[index - 1] != 'र' and character == '्' and unicodetext[index + 1] == 'र':
                    # for खुट्टा चिर्ने चिन्ह in the likes of क्रम and ट्रक
                    if unicodetext[index - 1] != 'ट' and unicodetext[index - 1] != 'ठ' and unicodetext[index-1] != 'ड':
                        normalized += '|'  # for sign as in क्रम
                        index += 1
                        continue
                    else:
                        normalized += '«'  # for sign as in ट्रक
                        index += 1
                        continue
            except IndexError:
                pass
            normalized += character
        except KeyError:
            normalized += character
    normalized = normalized.replace('त|', 'q')  # for त्र
    return normalized


def convert(unicodestring):
    normalizedunicodetext = normalizeUnicode(unicodestring)
    converted = ''
    index = -1
    while index + 1 < len(normalizedunicodetext):
        index += 1
        character = normalizedunicodetext[index]
        if character == '\ufeff':
            continue
        try:
            try:
                if normalizedunicodetext[index + 1] == 'ि':  # for normal hraswo ukaar
                    if character == 'q':
                        converted += 'l' + character
                    else:
                        converted += 'l' + unicodeToPreetiDict[character]
                    index += 1
                    continue

                if normalizedunicodetext[index + 2] == 'ि':  # for constructs like त्ति
                    if character in list('WERTYUXASDGHJK:ZVN'):
                        if normalizedunicodetext[index + 1] != 'q':  # if not like न्त्रि
                            converted += 'l' + character + unicodeToPreetiDict[normalizedunicodetext[index + 1]]
                            index += 2
                            continue
                        elif normalizedunicodetext[index + 1] == 'q':
                            converted += 'l' + character + normalizedunicodetext[index + 1]
                            index += 2
                            continue

                if normalizedunicodetext[index + 1] == '्' and character == 'र':  # for reph as in वार्ता
                    if normalizedunicodetext[index + 3] == 'ा' or normalizedunicodetext[index + 3] == 'ो' or \
                            normalizedunicodetext[index + 3] == 'ौ' or normalizedunicodetext[index + 3] == 'े' or \
                            normalizedunicodetext[index + 3] == 'ै' or normalizedunicodetext[index + 3] == 'ी':
                        converted += unicodeToPreetiDict[normalizedunicodetext[index + 2]] + unicodeToPreetiDict[
                            normalizedunicodetext[index + 3]] + '{'
                        index += 3
                        continue
                    elif normalizedunicodetext[index + 3] == 'ि':
                        converted += unicodeToPreetiDict[normalizedunicodetext[index + 3]] + unicodeToPreetiDict[
                            normalizedunicodetext[index + 2]] + '{'
                        index += 3
                        continue
                    converted += unicodeToPreetiDict[normalizedunicodetext[index + 2]] + '{'
                    index += 2
                    continue

                if normalizedunicodetext[index + 3] == 'ि':  # for the likes of ष्ट्रिय
                    if normalizedunicodetext[index + 2] == '|' or normalizedunicodetext[index + 2] == '«':
                        if character in list('WERTYUXASDGHJK:ZVNIi'):
                            converted += 'l' + character + unicodeToPreetiDict[normalizedunicodetext[index + 1]] + \
                                         normalizedunicodetext[index + 2]
                            index += 3
                            continue

            except IndexError:
                pass
            converted += unicodeToPreetiDict[character]
        except KeyError:
            converted += character

    converted = converted.replace('Si', 'I')  # Si in preeti is aadha ka aadha ष, so replace with I which is aadha क्ष
    converted = converted.replace('H`', '1')  # H` is the product of composite nature of unicode ज्ञ
    converted = converted.replace('b\w', '4')  # b\w means in preeti द halanta ध, so replace the composite
    converted = converted.replace('z|', '>')  # composite for श्र
    converted = converted.replace("/'", '?')  # composite for रु
    converted = converted.replace('/"', '¿')  # composite for रू
    converted = converted.replace('Tt', 'Q')  # composite for त्त
    converted = converted.replace('b\lj', 'lå')  # composite for द्वि
    converted = converted.replace('b\j', 'å')  # composite for द्व
    converted = converted.replace('0f\\', '0')  # composite for ण् to get the aadha ण in say गण्डक
    converted = converted.replace('`\\', '~')  # composite for aadha ञ्
    return converted



from xml.etree import ElementTree as ET
from io import StringIO
from .fontmapper import FontMapper
from .exceptions import UnsupportedMapToException
import os
import zipfile
import tempfile
import shutil


class DocxHandler:

    def __init__(self, rules_file, default_unicode_font_name="Kalimati"):
        self.mapper = FontMapper(rules_file)
        self.supported_ttf_fonts = self.mapper.supported_maps
        self.default_unicode_font_name = default_unicode_font_name
        self.known_devanagari_unicode_fonts = ["Kalimati", "Mangal", "Noto Sans Devanagari"]

    @staticmethod
    def __get_xml(docx_file_path):
        with zipfile.ZipFile(docx_file_path) as zf:
            with zf.open('word/document.xml') as xf:
                xml_content = xf.read().decode('utf-8')
        return xml_content

    @staticmethod
    def __save_docx(xml_content, output_file_path, original_file_path):
        tmp_dir = tempfile.mkdtemp()
        with zipfile.ZipFile(original_file_path, 'r') as zipObj:
            zipObj.extractall(tmp_dir)
            filenames = zipObj.namelist()
        with open(os.path.join(tmp_dir, 'word/document.xml'), 'w', encoding="utf-8") as f:
            f.write(xml_content)
        with zipfile.ZipFile(output_file_path, "w") as docx:
            for filename in filenames:
                docx.write(os.path.join(tmp_dir, filename), filename)
        shutil.rmtree(tmp_dir)
        return True

    @staticmethod
    def __register_namespaces(xml):
        namespaces = dict([node for _, node in ET.iterparse(xml, events=['start-ns'])])
        for ns in namespaces:
            ET.register_namespace(ns, namespaces[ns])

    @staticmethod
    def __get_font_data_from_relation_property(relation_property):
        used_font = "dummyFontThatWillNeverBeUsed"
        used_unicode_font = "dummyFontThatWillNeverBeUsed"
        font_property = relation_property.find(
            "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}rFonts")
        if font_property is not None:
            try:
                used_font = font_property.attrib[
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}ascii"
                ]
            except KeyError:
                used_unicode_font = font_property.attrib[
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}cs"
                ]
                used_font = "Unicode"
        return font_property, used_font, used_unicode_font

    def __map_now(self, map_to, font_property, text_container=None, used_font=None):
        # Strip un necessary font related attributes. Make it minimal.. Word processor
        # will add them back if necessary
        font_property.attrib = self.__strip_font_attributes(font_property.attrib)
        if map_to.lower() == "preeti":
            font_property.attrib[
                "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}ascii"
            ] = "Preeti"
            font_property.attrib[
                "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}hAnsi"
            ] = "Preeti"
            # Replace original text with mapped text
            if text_container is not None:
                original_text = text_container.text
                text_container.text = self.mapper.map_to_preeti(original_text, used_font, unescape_html_input=True)
        elif map_to.lower() == "unicode":
            font_property.attrib[
                "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}cs"
            ] = self.default_unicode_font_name
            # Replace original text with mapped text
            if text_container is not None:
                original_text = text_container.text
                text_container.text = self.mapper.map_to_unicode(original_text, used_font, unescape_html_input=True)
        else:
            raise UnsupportedMapToException("Document cannot be mapped to target ")

    @staticmethod
    def __strip_font_attributes(dictionary):
        # Don't know why but removing cstheme and eastAsiaTheme does magic and fixes map to incorrect font
        # So yes.... This is needed
        return {key: value for key, value in dictionary.items()
                if key not in [
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}cstheme",
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}eastAsiaTheme",
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}ascii",
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}hAnsi",
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}cs"
                ]}

    def __handle_wp_containers_in_paragraphs(self, paragraphs, from_font="auto", to_font="unicode",
                                             known_unicode_fonts=None):
        if known_unicode_fonts is None:
            known_unicode_fonts = []
        for paragraph in paragraphs:
            for relation in paragraph.iterfind("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}r"):
                relation_property = relation.find("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}rPr")
                if relation_property is not None:
                    font_property, used_font, used_unicode_font = self.__get_font_data_from_relation_property(
                        relation_property
                    )
                    text_container = relation.find("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}t")
                    if text_container is not None:
                        continue_mapping = True
                        set_from_font = None
                        if from_font == "auto":
                            # Check if the used font is in supported ttf fonts + "unicode"
                            if used_font in self.supported_ttf_fonts:
                                set_from_font = used_font
                                if to_font.lower() == "preeti":
                                    if used_unicode_font not in (self.known_devanagari_unicode_fonts +
                                                                 known_unicode_fonts + self.supported_ttf_fonts):
                                        continue_mapping = False
                            else:
                                continue_mapping = False
                        else:
                            # Since this is manual mode.. Do no checks and set used font to the font set to "from_font"
                            # variable. This will change font of whole document to "to_font", every text content will
                            # be interpreted as "from_font" and will be mapped to "to_font"
                            # Exception will be thrown while trying to map from/to unsupported fonts
                            set_from_font = from_font
                        # All checks must have been done. Now start mapping the fonts
                        if continue_mapping:
                            self.__map_now(to_font, font_property,
                                           text_container=text_container,
                                           used_font=set_from_font)

            # A "Not so quick" fix for pPr inside w:p inside "w:txbxcontent"
            for paragraph_property in paragraph.iterfind(
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}pPr"):
                relation_property = paragraph_property.find(
                    "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}rPr")
                if relation_property is not None:
                    font_property, used_font, used_unicode_font = self.__get_font_data_from_relation_property(
                        relation_property
                    )
                    continue_mapping = True
                    # If on auto auto mode, only change base font when mapping is possible
                    if from_font == "auto":
                        if used_font in self.supported_ttf_fonts:
                            if to_font.lower() == "preeti":
                                if used_unicode_font not in (self.known_devanagari_unicode_fonts +
                                                             known_unicode_fonts + self.supported_ttf_fonts):
                                    continue_mapping = False
                        else:
                            continue_mapping = False
                    if continue_mapping:
                        self.__map_now(to_font, font_property)
        return True

    def detect_used_fonts(self, docx_file_path):
        detected_supported_fonts = []

        tree = ET.ElementTree(ET.fromstring(self.__get_xml(docx_file_path)))
        root = tree.getroot()

        font_property_containers = root.iter("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}rFonts")
        for container in font_property_containers:
            used_font = container.attrib.get("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}ascii",
                                             "SomeUnsupportedFonts6576")
            if used_font in self.supported_ttf_fonts and used_font not in detected_supported_fonts:
                detected_supported_fonts.append(used_font)
        return detected_supported_fonts

    def map_fonts(self, original_file_path, output_file_path="mapped.docx", from_font="auto", to_font="unicode",
                  components=None, known_unicode_fonts=None):
        if known_unicode_fonts is None:
            known_unicode_fonts = []
        if components is None:
            components = ["body_paragraph", "table", "shape"]
        xml_string = self.__get_xml(original_file_path)
        tree = ET.ElementTree(ET.fromstring(xml_string))
        root = tree.getroot()
        # GET INSIDE DA BODIE
        for body in root.iterfind("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}body"):
            # Process normal paragraphs. They (w:p) lie directly inside body as child
            general_paragraphs = body.findall("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}p")
            if "body_paragraph" in components:
                self.__handle_wp_containers_in_paragraphs(general_paragraphs,
                                                          from_font=from_font,
                                                          to_font=to_font,
                                                          known_unicode_fonts=known_unicode_fonts)

            # Process paragraphs. They (w:p) lie inside "w:tbl" as child of table row and column, but we can just
            # iterate inside table to get them
            if "table" in components:
                for table in body.iterfind("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}tbl"):
                    # Now since we are not direct child of normal paragraphs but inside a table, We can use .iter()
                    # to get all the paragraphs inside current table without modifying other components
                    paragraphs = table.iter("{http://schemas.openxmlformats.org/wordprocessingml/2006/main}p")
                    self.__handle_wp_containers_in_paragraphs(paragraphs,
                                                              from_font=from_font,
                                                              to_font=to_font,
                                                              known_unicode_fonts=known_unicode_fonts)

            # Process shapes. They lie inside "w:p" (Main paragraphs), find them. Shapes wont be processed in
            # "body_paragraph" and SHOULD NOT BE as content in shape lie much deeper
            if "shape" in components:
                for paragraph in general_paragraphs:
                    tbx_content = paragraph.iter(
                        "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}txbxContent")
                    # They contain  data for (t)e(x)t(b)o(x)/Shapes. They content another "w:p" which has actual
                    # text content
                    for txbx in tbx_content:
                        paragraphs = txbx.iterfind(
                            "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}p")
                        self.__handle_wp_containers_in_paragraphs(paragraphs,
                                                                  from_font=from_font,
                                                                  to_font=to_font,
                                                                  known_unicode_fonts=known_unicode_fonts)

        # Make a temp directory for working, extract the original docx file in that temp directory
        # Replace content of document.xml in temporary directory with modified xml data . Repack everything back
        # And save the resulting package with the passed filename

        tmp_dir = tempfile.mkdtemp()
        self.__register_namespaces(StringIO(xml_string))
        tree.write(os.path.join(tmp_dir, "document.xml"), encoding="utf-8", xml_declaration=True)
        with open(os.path.join(tmp_dir, "document.xml"), "r") as f:
            xml_content = f.read()
        self.__save_docx(xml_content, output_file_path, original_file_path)
        shutil.rmtree(tmp_dir)



import argparse
import json
import os
import zipfile
from .base.docxhandler import DocxHandler
from .base.exceptions import *
from .base.fontmapper import FontMapper
from .base.txthandler import TxtHandler


def main():
    about = """ 
    Created by : Sabin Acharya (@trippygeese on github)
    License    : GNU GENERAL PUBLIC LICENSE v3
    Version    : 0.3.6
    Email      : sabin2059@protonmail.com
    """
    modes = ['string', 'plain', 'docx']
    parser = argparse.ArgumentParser(description=about, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-V', '--version', action='version', version="0.1a")
    parser.add_argument('-m', '--mode', dest='mode', help='Conversion mode ', choices=modes, required=True)
    parser.add_argument('-if', '--input-font', dest='font',
                        help='Font used in input file. ("auto" can be used for docx mode)',
                        default='preeti', required=True)
    parser.add_argument('-of', '--output-font', dest='outputfont',
                        help='Font to which output will be mapped to. (If unspecified output font will be set to '
                             'unicode)',
                        default='unicode', choices=["Preeti", "unicode"], required=False)
    parser.add_argument('-dc', '--docx-components', dest='docxcomponents',
                        help='Component of docx which will be processed. (Comma separated) Available: '
                             '"body_paragraph,table,shape" (If not specified all components will be processed)',
                        default='body_paragraph,table,shape', required=False)
    parser.add_argument('-kf', '--known-unicode-fonts', dest='knownunicodefonts',
                        help='Fonts to add to known supported unicode fonts while converting to preeti (If '
                             'Unspecified "Kalimati,Mangal,Noto Sans Devanagari" will be set)',
                        default='', required=False)
    parser.add_argument('-i', '--input', dest='input', help='Input string or filepath', required=True)
    parser.add_argument('-o', '--output', dest='output', help='Output file path. Not required for string mode')
    parser.add_argument('-mf', '--map-file', dest='mapfile', help='Mapping definition file')
    args = parser.parse_args()
    font = args.font
    op_mode = args.mode

    def splitnclean(string):
        lis = string.split(",")
        for index, item in enumerate(lis):
            lis[index] = item.strip()
        return lis

    rule_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "map.json")
    if args.mapfile is not None:
        rule_file = args.mapfile
    if op_mode == "string":
        try:
            converter = FontMapper(rule_file)
            if args.outputfont == 'unicode':
                print(converter.map_to_unicode(args.input, from_font=args.font))
            elif args.outputfont == 'Preeti':
                print(converter.map_to_preeti(args.input, from_font=args.font))
            else:
                raise UnsupportedMapToException
        except NoMapForOriginException:
            print("The mapping for selected origin font does not exist")
        except FileNotFoundError:
            print("Mapping definition file cannot be opened or does not exist.")
        except json.decoder.JSONDecodeError:
            print("Invalid mapping definition file")
        except UnsupportedMapToException:
            print("Cannot map to specified output font")
        except Exception as e:
            print("Unexpected error... Exiting !  " + str(e))
    elif op_mode == "plain" or op_mode == "docx":
        converter = None
        try:
            if op_mode == "plain":
                converter = TxtHandler(rule_file)
            elif op_mode == "docx":
                converter = DocxHandler(rule_file)
            converter.map_fonts(original_file_path=args.input, output_file_path=args.output, from_font=args.font,
                                to_font=args.outputfont, components=splitnclean(args.docxcomponents),
                                known_unicode_fonts=splitnclean(args.knownunicodefonts))
            print("The converted file is saved as :" + args.output)
        except NoMapForOriginException:
            print("The mapping for selected origin font does not exist")
        except FileNotFoundError:
            print("Mapping definition file cannot be opened or does not exist.")
        except json.decoder.JSONDecodeError:
            print("Invalid mapping definition file")
        except UnsupportedMapToException:
            print("Cannot map to given output font !")
        except TxtAutoModeException:
            print("Font autodetect does not work on txt files :(")
        except zipfile.BadZipFile:
            print("Improper docx file")
        # except Exception as e:
        #    print("Unexpected error... Exiting !  "+str(e))
    else:
        print("Unsupported operation mode")
